package edu.wpi.ari.glados.ins;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Vector;

import edu.wpi.ari.glados.insserver.INSDataPacket;
import edu.wpi.ari.glados.network.SensorConnection;

class INSConnector extends SensorConnection {
	
	private DataInputStream ois = null;
	
	private Vector<INSNetObserver> listeners;
	
	/**
	 * Default Constructor
	 * 
	 * @param host the host to connect to
	 * @param port the port to communicate across
	 */
	public INSConnector(String host, int port) {
		listeners = new Vector<INSNetObserver>();
		setHost(host);
		setPort(port);
	}
	
	/**
	 * Connect to the host and start the data collection thread
	 */
	public void connect() {
		try {
			super.connect();
			ois = new DataInputStream(getConnection().getInputStream());
			
			start();
		} catch (IOException e) {
			System.err.println("Error: Could not connect INS to " + getHost());
		}
		
	}

	/**
	 * Disconnect from the host
	 */
	public void disconnect() {
		try {
			ois.close();
			super.disconnect();
		} catch (IOException e) {
			
		}
	}

	/**
	 * Called by the thread
	 * 
	 * Constantly polls for data and then fires a update to all INSNetObservers
	 */
	public void run() {
		try {
			// poll for data
			while (true) {
				
				long t = ois.readLong();
				double x = ois.readDouble();
				double y = ois.readDouble();
				double z = ois.readDouble();
				int w = ois.readInt();
				
				fireUpdate(t,x,y,z,w);
			}
		} catch (IOException e) {
			disconnect();
			reconnect();
			System.err.println(e.getMessage());
		} finally {
			System.out.println("WTF!");
		}
	}
	
	/**
	 * Reconnect to the host
	 */
	private void reconnect() {
		try {
			Thread.sleep(3000);
			run();
		} catch (InterruptedException e) {

		}
	}

	/**
	 * Update all of the observers when there is new data
	 * @param dp
	 */
	private void fireUpdate(long time, double x, double y, double z, int yaw) {
		for(INSNetObserver l : listeners) {
			l.update(time, x, y, z, yaw);
		}
	}
	
	/**
	 * Add an observer 
	 * 
	 * @param l the observer to add to the list
	 */
	public void addListener(INSNetObserver l) {
		listeners.add(l);
	}
	
	/**
	 * Remove an observer
	 * 
	 * @param l the observer to remove from the list
	 */
	public void removeListener(INSNetObserver l) {
		listeners.remove(l);
	}
	
	/**
	 * Remove all observers from the list
	 */
	public void removeAll() {
		listeners.clear();
	}
	
}
