package edu.wpi.ari.glados.slam;

import java.io.Serializable;

import Jama.Matrix;
import javax.vecmath.Point3d;

public class Control implements Stim, Serializable{
	
	private static final long serialVersionUID = 1L;
	/**
	 * 6-element column vector - x y z rx ry rz
	 */
	private Point3d control;
	private Matrix rot;
	
	public Control(){
		this.control = new Point3d (0., 0., 0.);
		this.rot = new Matrix (3, 1, 0);
	}
	
	public Control(double x, double y, double z, double dx, double dy, double dz){
		final double[] c = {dx, dy, dz};
		this.control = new Point3d(x, y, z);
		this.rot = new Matrix(c, 3);
	}
	
	public String toString(){
		return "Pos Change: " + this.control.toString() + 
        	" lin (" + this.rot.get(0, 0) +
        	", " + this.rot.get(1, 0) + 
        	", " + this.rot.get(2, 0) + ")rad";	
	}
	
	public double getXRot(){
		return this.rot.get(0, 0);
	}
	
	public double getYRot(){
		return this.rot.get(1, 0);
	}
	
	public double getZRot(){
		return this.rot.get(2, 0);
	}

	public Point3d getControl() {
		return control;
	}

	public void setControl(Point3d control) {
		this.control = control;
	}

	public Matrix getRot() {
		return rot;
	}

	public void setRot(Matrix rot) {
		this.rot = rot;
	}
	
	public void addControl(Control ctrl){
		this.control.add(ctrl.control);
		this.rot.plus(ctrl.rot);
	}
	
	

}
