package edu.wpi.ari.glados.slam;

import java.io.Serializable;
import java.util.Random;

import Jama.Matrix;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

/**
 * Dual-purpose structure for both global and local landmarks
 * Contains a 3D point
 * @author Alex Cunningham
 *
 */
public class Landmark implements Serializable{

	private static final long serialVersionUID = 1L;

	//private Matrix mean;
	private Matrix cov;
	
	private Point3d mean;
	
	private int score;

	
	public Landmark(){
		//create a blank landmark to update - use for non-truth
		this.mean = new Point3d();
		this.cov = new Matrix(3, 3, 0);
		this.score = 0;
	}
	
	public Landmark(Point3d mean){
		this.mean = new Point3d(mean.x, mean.y, mean.z);
		this.score = 0;
	}
	
	public Landmark(Vector3d mean){
		this.mean = new Point3d(mean.x, mean.y, mean.z);
		this.score = 0;
	}
	
	public Landmark(double x, double y, double z){
		this.mean = new Point3d(x, y, z);
		this.score = 0;
	}
	
	public static Landmark randLandmark(double scale, double offset){
		Random rand = new Random();
		return new Landmark(scale*rand.nextDouble()-offset, 
				            scale*rand.nextDouble()-offset,
				            scale*rand.nextDouble()-offset);
	}

	public Point3d getMean() {
		return mean;
	}

	public void setMean(Point3d mean) {
		this.mean = mean;
	}

	public Matrix getCov() {
		return cov;
	}

	public void setCov(Matrix cov) {
		this.cov = cov;
	}

	public double dist3D(Landmark landmark) {
	
		return this.mean.distance(landmark.getMean());
	}
	
	public String toString(){
		return this.mean.toString();
	}
	
	public boolean meanEqual(Landmark lm){
		return false;
	}
	
	public double getX(){
		return this.mean.x;
	}

	public double getY(){
		return this.mean.y;
	}
	
	public double getZ(){
		return this.mean.z;
	}
	
	public void setX(double x){
		this.mean.x = x;
	}
	
	public void setY(double y){
		this.mean.y = y;
	}
	
	public void setZ(double z){
		this.mean.z = z;
	}

	public int getScore() {
		return score;
	}

	public void setScore(int score) {
		this.score = score;
	}
	
	public void incScore(){
		this.score += 1;
	}
	
	public boolean equals(Landmark lm){
		return this.mean.x == lm.getX() &&
		       this.mean.y == lm.getY() &&
		       this.mean.z == lm.getZ();
	}

	public boolean equals(Landmark lm, double d) {
		return Math.abs(Math.abs(this.mean.x)- Math.abs(lm.getX())) < d &&
			   Math.abs(Math.abs(this.mean.y)- Math.abs(lm.getY())) < d  &&
			   Math.abs(Math.abs(this.mean.z)- Math.abs(lm.getZ())) < d ;
	}

	public void scale(double scale) {
		this.mean.scale(scale);		
	}
	
	
}
