package edu.wpi.ari.glados.slam;

import java.io.Serializable;
import java.util.Vector;

/**
 * Container class for landmarks that form a map of points
 * This will eventually include a number of searching/sorting
 * methods for more efficient searching structures
 * @author Alex Cunningham
 *
 */
public class PointMap implements Stim, Serializable{
	
	private static final long serialVersionUID = 1L;
	
	private Vector<Landmark> landmarks;
	
	public PointMap(){
		this.landmarks = new Vector<Landmark>();
	}
		
	public int getNumLandmarks(){
		return this.landmarks.size();
	}

	public void addLandmark(Landmark lm){
		this.landmarks.add(lm);
	}
	
	public void addLandmarks(Vector<Landmark> lms){
		this.landmarks.addAll(lms);
	}
	
	public Landmark getLandmark(int id){
		return this.landmarks.get(id);
	}
	
	public void rmLandmark(Landmark lm){
		this.landmarks.remove(lm);
	}
	
	public void clearLandmarks(){
		this.landmarks.clear();
	}
	
	public Vector<Landmark> getLandmarks() {
		return landmarks;
	}

	public void setLandmarks(Vector<Landmark> landmarks) {
		this.landmarks = landmarks;
	}

	public void addMap(PointMap map){
		this.landmarks.addAll(map.getLandmarks());
	}
	
	public String toString(){
		String ret = "";
		ret = ret + "  Contains " + this.landmarks.size() + " landmarks\n";
		for (int i = 0; i < this.landmarks.size(); i++){
			ret = ret + "     " + this.landmarks.get(i).toString() + "\n";
		}
		return ret;
	}

	public void scale(double scale) {
		for (int i = 0; i < this.getNumLandmarks(); i++){
			this.landmarks.get(i).scale(scale);
		}
		
	}

}
