package edu.wpi.ari.glados.slam;

import Jama.Matrix;
import javax.vecmath.Point3d;

public class Pose {
	
	/**
	 * 6-element column vector, x y z rx ry rz
	 */
	private Point3d linPose;
	private Matrix rotPose;
	
	/**
	 * Creates a generic pose located at the origin
	 */
	public Pose(){
		this.linPose = new Point3d (0, 0, 0);
		this.rotPose = new Matrix (3, 1, 0);
	}
	
	public Pose(double x, double y, double z, double x_rot, double y_rot, double z_rot){
		this.linPose = new Point3d(x, y, z);
		this.rotPose = new Matrix (3, 1);
		this.rotPose.set(0, 0, x_rot);
		this.rotPose.set(1, 0, y_rot);
		this.rotPose.set(2, 0, z_rot);
	}
	
	public static Pose clone(Pose p){
		return new Pose(p.getX(), p.getY(), p.getZ(), p.getXRot(), p.getYRot(), p.getZRot());
	}

	public void addControl(Control control) {
		this.linPose.add(control.getControl());		
		this.rotPose.plus(control.getRot());
	}
	
	public String toString(){
		return this.linPose.toString() + 
		                   " lin (" + this.rotPose.get(0, 0) +
		                   ", " + this.rotPose.get(1, 0) + 
		                   ", " + this.rotPose.get(2, 0) + ")rad";
	}

	public double getXRot() {
		return this.rotPose.get(0, 0);
	}

	public double getYRot() {
		return this.rotPose.get(1, 0);
	}

	public double getZRot() {
		return this.rotPose.get(2, 0);
	}
	
	public double getX() {
		return this.linPose.x;
	}

	public double getY() {
		return this.linPose.y;
	}

	public double getZ() {
		return this.linPose.z;
	}

	public Point3d getLinPose() {
		return linPose;
	}

	public void setLinPose(Point3d linPose) {
		this.linPose = linPose;
	}

	public Matrix getRotPose() {
		return rotPose;
	}

	public void setRotPose(Matrix rotPose) {
		this.rotPose = rotPose;
	}
	
	public boolean equals(Pose pose){
		return this.linPose.x == pose.linPose.x &&
			   this.linPose.y == pose.linPose.y &&
			   this.linPose.z == pose.linPose.z &&
			   this.getXRot() == pose.getXRot() &&
			   this.getYRot() == pose.getYRot() &&
			   this.getZRot() == pose.getZRot();
	}

	public void scale(double scale) {
		this.linPose.scale(scale);
		this.rotPose.times(scale);
	}	
	
	
	
	
	

}
