package edu.wpi.ari.glados.slam;

import java.awt.Color;

/**
 * Simple Container class for color schemes for the SLAM 
 * Visualization Module.  Provides configuration and 
 * defaults.
 * @author Alex Cunningham
 *
 */
public class SLAMVisColorScheme {
	
	private Color bkgnd;
	private Color axis_lines;
	private Color past_poses;
	private Color cur_pose;
	private Color landmarks;
	private Color text;
	
	public SLAMVisColorScheme(){
		bkgnd = Color.black;
		axis_lines = Color.gray;
		past_poses = Color.lightGray;
		cur_pose = Color.magenta;
		landmarks = Color.blue;
		text = Color.white;
	}
	
	/**
	 * Return a default color scheme for screen purposes
	 * Uses dark backgrounds which are unsuitable for text
	 * @return
	 */
	public static SLAMVisColorScheme getScreenDefault(){
		SLAMVisColorScheme scheme = new SLAMVisColorScheme();
		scheme.bkgnd = Color.black;
		scheme.axis_lines = Color.gray;
		scheme.past_poses = Color.lightGray;
		scheme.cur_pose = Color.magenta;
		scheme.landmarks = Color.blue;
		scheme.text = Color.white;		
		return scheme;
	}

	
	/**
	 * Return a print-capable color scheme
	 * @return
	 */
	public static SLAMVisColorScheme getPrintDefault(){
		SLAMVisColorScheme scheme = new SLAMVisColorScheme();
		scheme.bkgnd = Color.white;
		scheme.axis_lines = Color.DARK_GRAY;
		scheme.past_poses = Color.lightGray;
		scheme.cur_pose = Color.magenta;
		scheme.landmarks = Color.blue;
		scheme.text = Color.black;		
		return scheme;
	}
	
	// Basic Set of Get/Set functions
	
	public Color getBkgnd() {
		return bkgnd;
	}

	public void setBkgnd(Color bkgnd) {
		this.bkgnd = bkgnd;
	}

	public Color getAxis_lines() {
		return axis_lines;
	}

	public void setAxis_lines(Color axis_lines) {
		this.axis_lines = axis_lines;
	}

	public Color getPast_poses() {
		return past_poses;
	}

	public void setPast_poses(Color past_poses) {
		this.past_poses = past_poses;
	}

	public Color getCur_pose() {
		return cur_pose;
	}

	public void setCur_pose(Color cur_pose) {
		this.cur_pose = cur_pose;
	}

	public Color getLandmarks() {
		return landmarks;
	}

	public void setLandmarks(Color landmarks) {
		this.landmarks = landmarks;
	}

	public Color getText() {
		return text;
	}

	public void setText(Color text) {
		this.text = text;
	}
	
	public boolean equals(SLAMVisColorScheme sl){
		return this.bkgnd == sl.bkgnd &&
			   this.axis_lines == sl.axis_lines &&
			   this.past_poses == sl.past_poses &&
			   this.cur_pose == sl.cur_pose &&
			   this.landmarks == sl.landmarks &&
			   this.text == sl.text;
	}
	
}
