package edu.wpi.ari.glados.slam.slamIDE;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerListModel;

import edu.wpi.ari.glados.slam.Landmark;
import edu.wpi.ari.glados.slam.PointMap;

/**
 * Singleton Class that acts as a filter for landmarks to switch
 * coordinate systems around. 
 * @author Alex Cunningham
 *
 */
public class FixCoords extends JFrame implements ActionListener {

	private static final long serialVersionUID = 4094453438307031913L;

	
	private static FixCoords self;
	private enum Options {X_POS, X_NEG, Y_POS, Y_NEG, Z_POS, Z_NEG};
	private final String[] opt_strings = {"+X", "-X", "+Y", "-Y", "+Z", "-Z"};  
	private Options env_x, env_y, env_z, ctrl_x, ctrl_y, ctrl_z; 
	
	//GUI Components
	private JSpinner spin_env_x, spin_env_y, spin_env_z, spin_ctrl_x, spin_ctrl_y, spin_ctrl_z; 
	private JButton save, cancel;
	
	
	public static FixCoords getFixCoords(){
		if (FixCoords.self == null){
			FixCoords.self = new FixCoords();
		}
		return FixCoords.self;
	}
	
	private FixCoords(){
		this.setLayout(new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));
		
		//Initialize filter to defaults
		env_x = Options.X_POS;
		env_y = Options.Y_POS;
		env_z = Options.Z_POS;
		ctrl_x = Options.X_POS;
		ctrl_y = Options.Y_POS;
		ctrl_z = Options.Z_POS;	

		//Add fields and buttons
		SpinnerListModel coordsel1 = new SpinnerListModel(this.opt_strings);
		spin_env_x = new JSpinner(coordsel1);
		spin_env_x.setValue(this.opt_strings[0]);
		JLabel label1 = new JLabel("ENV - X");
		label1.setLabelFor(spin_env_x);
		add(spin_env_x);
		add(label1);
		
		SpinnerListModel coordsel2 = new SpinnerListModel(this.opt_strings);
		spin_env_y = new JSpinner(coordsel2);
		spin_env_y.setValue(this.opt_strings[2]);
		JLabel label2 = new JLabel("ENV - Y");
		label2.setLabelFor(spin_env_y);
		add(spin_env_y);
		add(label2);
		
		SpinnerListModel coordsel3 = new SpinnerListModel(this.opt_strings);
		spin_env_z = new JSpinner(coordsel3);
		spin_env_z.setValue(this.opt_strings[4]);
		JLabel label3 = new JLabel("ENV - Z");
		label3.setLabelFor(spin_env_z);
		add(spin_env_z);
		add(label3);
		
		SpinnerListModel coordsel4 = new SpinnerListModel(this.opt_strings);
		spin_ctrl_x = new JSpinner(coordsel4);
		spin_ctrl_x.setValue(this.opt_strings[0]);
		JLabel label4 = new JLabel("CTRL - X");
		label4.setLabelFor(spin_ctrl_x);
		add(spin_ctrl_x);
		add(label4);
		
		
		SpinnerListModel coordsel5 = new SpinnerListModel(this.opt_strings);
		spin_ctrl_y = new JSpinner(coordsel5);
		spin_ctrl_y.setValue(this.opt_strings[2]);
		JLabel label5 = new JLabel("CTRL - Y");
		label5.setLabelFor(spin_ctrl_y);
		add(spin_ctrl_y);
		add(label5);
		
		SpinnerListModel coordsel6 = new SpinnerListModel(this.opt_strings);
		spin_ctrl_z = new JSpinner(coordsel6);
		spin_ctrl_z.setValue(this.opt_strings[4]);
		JLabel label6 = new JLabel("CTRL - Z");
		label6.setLabelFor(spin_ctrl_z);
		add(spin_ctrl_z);
		add(label6);
		
		//add control buttons
		this.save = new JButton("Save");
		this.save.addActionListener(this);
		this.save.setSize(150, 50);
		add(this.save);
		
		this.cancel = new JButton("Cancel");
		this.cancel.addActionListener(this);
		this.cancel.setSize(150, 50);
		add(this.cancel);
		
		//configure window
		this.setSize(200, 400);
		//this.setResizable(false);
		
	}
	
	public void showFixCoordsDialog(){
		//set visible inputs to current values
		this.spin_env_x.setValue(getSpinValue(this.env_x));
		this.spin_env_y.setValue(getSpinValue(this.env_y));
		this.spin_env_z.setValue(getSpinValue(this.env_z));
		this.spin_ctrl_x.setValue(getSpinValue(this.ctrl_x));
		this.spin_ctrl_y.setValue(getSpinValue(this.ctrl_y));
		this.spin_ctrl_z.setValue(getSpinValue(this.ctrl_z));
	
		this.setVisible(true);
	}
	


	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(save)){
			save();
		} else if (e.getSource().equals(cancel)){
			cancel();
		}
	}

	private void cancel() {
		setVisible(false);
	}

	/**
	 * Saves the data from the spinners to the filter
	 * 
	 * This save function does not include validation 
	 * of the inputs, so it is up the user to avoid 
	 * screwing up their system
	 */
	private void save() {
		this.env_x = getValue((String) this.spin_env_x.getValue());
		this.env_y = getValue((String) this.spin_env_y.getValue());
		this.env_z = getValue((String) this.spin_env_z.getValue());
		this.ctrl_x = getValue((String) this.spin_ctrl_x.getValue());
		this.ctrl_y = getValue((String) this.spin_ctrl_y.getValue());
		this.ctrl_z = getValue((String) this.spin_ctrl_z.getValue());
		
		this.setVisible(false);
	}
	
	private Options getValue(String s){
		if (s.equals(this.opt_strings[0])){
			return FixCoords.Options.X_POS;
		} else if (s.equals(this.opt_strings[1])){
			return FixCoords.Options.X_NEG;
		} else if (s.equals(this.opt_strings[2])){
			return FixCoords.Options.Y_POS;
		} else if (s.equals(this.opt_strings[3])){
			return FixCoords.Options.Y_NEG;
		} else if (s.equals(this.opt_strings[4])){
			return FixCoords.Options.Z_POS;
		} else if (s.equals(this.opt_strings[5])){
			return FixCoords.Options.Z_NEG;
		} 
		return null;
	}
	
	private String getSpinValue(Options o){
		if (o == Options.X_POS){
			return this.opt_strings[0];
		} else if (o == Options.X_NEG){
			return this.opt_strings[1];
		} else if (o == Options.Y_POS){
			return this.opt_strings[2];
		} else if (o == Options.Y_NEG){
			return this.opt_strings[3];
		} else if (o == Options.Z_POS){
			return this.opt_strings[4];
		} else if (o == Options.Z_NEG){
			return this.opt_strings[5];
		} 
		return null;
	}
	
	
	public void filter(Landmark lm){
		double x, y, z;
		x = lm.getX();
		y = lm.getY();
		z = lm.getZ();
		lm.setX(getNewX(x, y, z));
		lm.setY(getNewY(x, y, z));
		lm.setZ(getNewZ(x, y, z));
	}
	
	public void filter(PointMap m){
		for (int i = 0; i < m.getNumLandmarks(); i++){
			filter(m.getLandmark(i));
		}
	}
	
	private double getNewX(double x, double y, double z) {
		if (this.env_x == Options.X_POS){
			return x;
		} else if (this.env_x == Options.X_NEG){
			return x*(-1);
		} else if (this.env_x == Options.Y_POS){
			return y;
		} else if (this.env_x == Options.Y_NEG){
			return y*(-1);
		} else if (this.env_x == Options.Z_POS){
			return z;
		} else if (this.env_x == Options.Z_NEG){
			return z*(-1);
		} 
		return 1;
	}



	private double getNewY(double x, double y, double z) {
		if (this.env_y == Options.X_POS){
			return x;
		} else if (this.env_y == Options.X_NEG){
			return x*(-1);
		} else if (this.env_y == Options.Y_POS){
			return y;
		} else if (this.env_y == Options.Y_NEG){
			return y*(-1);
		} else if (this.env_y == Options.Z_POS){
			return z;
		} else if (this.env_y == Options.Z_NEG){
			return z*(-1);
		} 
		return 1;
	}

	private double getNewZ(double x, double y, double z) {
		if (this.env_z == Options.X_POS){
			return x;
		} else if (this.env_z == Options.X_NEG){
			return x*(-1);
		} else if (this.env_z == Options.Y_POS){
			return y;
		} else if (this.env_z == Options.Y_NEG){
			return y*(-1);
		} else if (this.env_z == Options.Z_POS){
			return z;
		} else if (this.env_z == Options.Z_NEG){
			return z*(-1);
		} 
		return 1;
	}



}
