package edu.wpi.ari.glados.slam.slamIDE;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JOptionPane;

import edu.wpi.ari.glados.slam.*;
import edu.wpi.ari.glados.slam.particle.Particle;

/**
 * Handles the playback of the datalog files, as well
 * as providing the GUI implementation.  
 * @author Alex Cunningham
 *
 */
public class PlaybackControls extends JPanel implements ActionListener{

	private static final long serialVersionUID = 1L;
	
	// Links to other GUI components
	private IDEFrame fr;
	
	//GUI Components
	private JButton clear, stepVis, stepSingle, 
					skipTo, skipToEnd, setScale, fixCoords;
	private JTextArea report;
	private JLabel stepIndicator;
	
	//Modules
	private FixCoords coordfilter;
	private ScaleFilter scalefilter;
	
	//Simulation information
	private int curPlace;
	private Particle particle;
	private Vector<Stim> data;
		
	public PlaybackControls(IDEFrame frame){
		
		this.fr = frame;
		this.setLayout(new FlowLayout());
		
		// add buttons
		
		this.clear = new JButton("Clear");
		this.clear.addActionListener(this);
		this.add(this.clear);
		
		this.stepVis = new JButton("Next Change");
		this.stepVis.addActionListener(this);
		this.add(this.stepVis);
		
		this.stepSingle = new JButton("Step");
		this.stepSingle.addActionListener(this);
		this.add(this.stepSingle);
		
		this.skipTo = new JButton("Skip To...");
		this.skipTo.addActionListener(this);
		this.add(this.skipTo);
		
		this.skipToEnd = new JButton("Skip to End");
		this.skipToEnd.addActionListener(this);
		this.add(this.skipToEnd);
		
		// setup reporting output
		this.report = new JTextArea(30, 22);
		this.report.setBackground(this.getBackground());
		this.report.setEditable(false);
		JScrollPane scroll = new JScrollPane(this.report);
		//scroll.setSize(this.getWidth()-10, this.getHeight()/2);
		this.add(scroll);
		
		// More buttons
		this.setScale = new JButton("Set Input Scaling");
		this.setScale.addActionListener(this);
		this.add(this.setScale);
		
		this.fixCoords = new JButton("Set Coordinates");
		this.fixCoords.addActionListener(this);
		this.add(this.fixCoords);
		
		//Indicator
		this.stepIndicator = new JLabel("Current Step: 0");
		this.add(this.stepIndicator);
		
		// setup simulation
		this.curPlace = 0;
		this.particle = this.fr.getParticle();
		this.data = new Vector<Stim>();
		
		// prepare filters
		this.coordfilter = FixCoords.getFixCoords();
		this.scalefilter = ScaleFilter.getScaleFilter();
		
		
	}


	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(this.clear)){
			clear();
		} else if (e.getSource().equals(this.stepVis)){
			stepVis();
		} else if (e.getSource().equals(this.stepSingle)){
			stepSingle();
		} else if (e.getSource().equals(this.skipTo)){
			skipTo();
		} else if (e.getSource().equals(this.skipToEnd)){
			skipToEnd();
		} else if (e.getSource().equals(this.setScale)){
			setScale();
		} else if (e.getSource().equals(this.fixCoords)){
			fixCoords();
		}
		this.updateIndicators();
	}

	/**
	 * Displays the coordinate configuration screen, which 
	 * allows for coordinate system switching for both controls
	 * and INS
	 */
	private void fixCoords() {
		this.coordfilter.showFixCoordsDialog();
	}

	/**
	 * Sets the input scaling factor used for the inputs, 
	 * with a single coefficient for each one
	 */
	private void setScale() {
		this.scalefilter.showDialog();
		
	}


	/**
	 * Resets the counter and Particle
	 */
	private void clear() {
		this.particle.clear();
		this.curPlace = 0;
		this.updateIndicators();
		fr.printToConsole("Particle Cleared");
		
	}


	/**
	 * Processes all of the stims until the end
	 */
	private void skipToEnd() {
		this.skipTo(this.data.size());	
	}
	
	/**
	 * Skips directly to a particular stim input by number
	 * If beyond the range of the dataset, goes to end
	 */
	private void skipTo() {
		String ret = (String) JOptionPane.showInputDialog(this.fr, "Input Stim Number", "Input", 
														  JOptionPane.PLAIN_MESSAGE, null, null, null);
		if (ret != null){
			int i = Integer.parseInt(ret);
			//if forward in sequence
			if (i > this.curPlace && i < this.data.size()){
				this.skipTo(i);
			//if i is 0 or negative, simply clears particle
			} else if (i <= 0){
				this.clear();
			//if i is before the current location, it clears and runs up to that point
			} else if (i > 0 && i < this.curPlace){
				this.clear();
				this.skipTo(i);
			//if going to end or beyond, just go to end
			} else if (i > this.data.size()){
				this.skipToEnd();
			}
		}
	}
	
	/**
	 * skips to a particular stim in the sequence
	 * This function is the base level function and 
	 * should be used underneath a wrapper as there
	 * is no range checking
	 * @param i
	 */
	private void skipTo(int index) {
		index = index - this.curPlace;
		for (int i = 0; i < index; i++){
			this.stepSingle();
		}
	}

	/**
	 * Processes only the next stim
	 */
	private void stepSingle() {
		if (!this.atEnd()){
			Stim s = this.getNextStim();
			this.particle.processStim(s);
		} else {
			this.fr.printToConsole("Step Aborted: At end of Datalog!");
		}
	}

	/**
	 * Gets the next stim in the vector, and increments the 
	 * step counter
	 * @return
	 */
	private Stim getNextStim() {
		if (this.data.size() >= this.curPlace){
			Stim s = this.data.get(this.curPlace);
			curPlace++;
			return s;
		} else {
			this.fr.printToConsole("Error!  Exceeded Bound on Datalog!");
			return null;
		}
	}


	/**
	 * Steps through to the next viewpoint update
	 * Displays error if reaches the end
	 */
	private void stepVis() {
		boolean foundVis = false;
		while (!foundVis){
			if (this.atVis() && !this.atEnd()){
				foundVis = true;
			} else if (this.atEnd()){
				return;
			}
			this.stepSingle();
		}
	}

	
	/**
	 * Writes information about the datalog to the 
	 * report window.  This function generates
	 * stats on the 
	 */
	public void logReport(){
		
	}


	public void setData(Vector<Stim> input) {
		this.data = input;
	}
	
	private void updateIndicators(){
		//update current step indicator
		this.stepIndicator.setText("Current Step: " + this.curPlace); 
		//repaint visualization
		this.fr.getSlamvis().paint(this.fr.getSlamvis().getGraphics());
	}
	
	/**
	 * Returns true if out of data
	 * @return
	 */
	private boolean atEnd(){
		return this.curPlace >= this.data.size();
	}
	
	/**
	 *  Returns true if current stim is a visible stim
	 * @return
	 */
	private boolean atVis(){
		return this.curPlace < this.data.size() && this.data.get(this.curPlace) instanceof PointMap;
	}


	public Vector<Stim> getData() {
		return this.data;
	}
	
	
}
