package edu.wpi.ari.glados.slam.slamIDE;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;

import edu.wpi.ari.glados.slam.Control;
import edu.wpi.ari.glados.slam.Landmark;
import edu.wpi.ari.glados.slam.PointMap;

/**
 * Singleton Filter Module that scales the 
 * inputs, and has a dialog box for user input
 * @author Alex Cunningham
 *
 */
public class ScaleFilter extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1350788660658801120L;
	
	private double lmScale;
	private double ctrlScale;
	
	//GUI Components
	JTextField landmarkScale, controlScale;
	JButton save, cancel;
	
	private static ScaleFilter filter;
	
	private ScaleFilter(){
		lmScale = 1;
		ctrlScale = 1;
		
		landmarkScale = new JTextField();
		JLabel label1 = new JLabel("Landmark Scale");
		this.add(this.landmarkScale);
		label1.setLabelFor(landmarkScale);
		this.add(label1);
		
		controlScale = new JTextField();
		JLabel label2 = new JLabel("Control Scale");
		this.add(this.controlScale);
		label2.setLabelFor(controlScale);
		this.add(label2);
		
		save = new JButton("Save");
		save.addActionListener(this);
		this.add(save);
		
		cancel = new JButton("Cancel");
		cancel.addActionListener(this);
		this.add(cancel);
	}
	
	
	public void showDialog() {
		
		this.setLayout(new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));
		this.setSize(200, 200);

		this.controlScale.setText(String.valueOf(this.ctrlScale));
		this.landmarkScale.setText(String.valueOf(this.lmScale));
		
		this.setVisible(true);
		
		
	}

	public static ScaleFilter getScaleFilter() {
		if (ScaleFilter.filter == null){
			ScaleFilter.filter = new ScaleFilter();
		}
		return ScaleFilter.filter;
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(save)){
			save();
		} else if (e.getSource().equals(cancel)){
			cancel();
		}
	}

	private void cancel() {
		setVisible(false);
	}

	/**
	 * 
	 */
	private void save() {
		setVisible(false);
		String temp = this.controlScale.getText();
		if (temp != null)
			this.ctrlScale = Double.parseDouble(temp);
		temp = this.landmarkScale.getText();
		if (temp != null)
			this.lmScale = Double.parseDouble(temp);
	}


	public double getLmScale() {
		return lmScale;
	}




	public void setLmScale(double lmScale) {
		this.lmScale = lmScale;
	}




	public double getCtrlScale() {
		return ctrlScale;
	}




	public void setCtrlScale(double ctrlScale) {
		this.ctrlScale = ctrlScale;
	}
	
	public void filter(Landmark lm){
		lm.getMean().scale(this.lmScale);
	}
	
	public void filter(PointMap map){
		for (int i = 0; i < map.getNumLandmarks(); i++){
			filter(map.getLandmark(i));
		}
	}
	
	public void filter(Control c){
		c.getControl().scale(this.ctrlScale);
	}
}
