package edu.wpi.ari.glados.vision;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;


public class ImageCanvas extends Canvas
{
	private static final long serialVersionUID = 1L;
	

	static final int CONNECTED = 0;
	static final int DISCONNECTED = 1;
	static final int STOPPED = 2;
	private double fps;
	private int connected;
	private boolean first_image;
	private Dimension preferredSize;
	private Graphics offGraphics;
	private Image currentImage;
	private Image offImage;
	private Frame mainWindow;
	private VisionConnection downloader;
	
	public ImageCanvas(String s, int i, int j, int k, int l)
	{
		connected = STOPPED;
		first_image = true;
		fps = j;
		preferredSize = new Dimension(k, l);
		File file = new File("images/ready.png");
		
		if(file.exists()) {
			setImage(Toolkit.getDefaultToolkit().getImage("images/ready.png"));
		} else {
			System.out.println("init.jpg not found");
		}

		downloader = new VisionConnection(s, i, j, this);
	}
	
	public void connect() {
		downloader.start();
		
		File file = new File("images/init.png");
		if(file.exists()) {
			setImage(Toolkit.getDefaultToolkit().getImage("images/init.png"));
		} else {
			System.out.println("init.jpg not found");
		}
		
		first_image = true;

	}

	public Dimension getPreferredSize()
	{
		return preferredSize;
	}

	public double getFPS()
	{
		return fps;
	}

	public void setMainWindow(Frame frame)
	{
		mainWindow = frame;
	}

	public void setImageSize(Dimension dimension)
	{
		preferredSize = dimension;
		setSize(dimension);
		setPreferredSize(dimension);
		setMinimumSize(dimension);
	}

	public int setImage(Image image)
	{
		if(first_image && mainWindow != null)
		{
			int i;
			int k;
			while((i = image.getWidth(null)) < 0 || (k = image.getHeight(null)) < 0) ;
			System.out.println("w = " + i + ", h = " + k);
			mainWindow.setBounds(100, 100, i, k);
			first_image = false;
		}
		int j = 0;
		byte byte0 = 5;
		while(!prepareImage(image, this)) 
		{
			wait(byte0);
			if(++j * byte0 > 5000)
				break;
		}
		currentImage = image;
		repaint();
		return j * byte0;
	}

	public void couldNotConnect()
	{

		File file = new File("images/error.png");
		if(file.exists())
		{
			setImage(Toolkit.getDefaultToolkit().getImage("images/error.png"));
			return;
		} else
		{
			System.out.println("error.jpg not found");
			return;
		}
	}

	public void setConnected(int i)
	{
		connected = i;
		repaint();
	}

	public void wait(int i)
	{
		try
		{
			Thread.sleep(i);
			return;
		}
		catch(InterruptedException _ex)
		{
			return;
		}
	}

	public void disconnect()
	{
		downloader.disconnect();
	}

	public void paint(Graphics g)
	{
		update(g);
	}

	public void update(Graphics g)
	{
		Color color = Color.black;
		if(currentImage != null)
		{
			offImage = createImage(preferredSize.width, preferredSize.height);
			offGraphics = offImage.getGraphics();
			offGraphics.drawImage(currentImage, 0, 0, preferredSize.width, preferredSize.height, this);
			g.drawImage(offImage, 0, 0, this);
			switch(connected)
			{
			case 0: // '\0'
				color = new Color(30, 175, 30);
				break;

			case 1: // '\001'
				color = new Color(175, 30, 30);
				break;

			case 2: // '\002'
				color = new Color(175, 30, 30);
				break;
			}
			g.setColor(Color.black);
			g.fillOval(preferredSize.width - 11, preferredSize.height - 11, 7, 7);
			g.setColor(color);
			g.fillOval(preferredSize.width - 12, preferredSize.height - 12, 7, 7);
		}
	}
	
	public BufferedImage getBufferedImage() {
		return downloader.getImage();
	}
}
