package edu.wpi.ari.glados.vision;

import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.PrintWriter;

import edu.wpi.ari.glados.network.SensorConnection;

public class VisionConnection extends SensorConnection
{
	
	private ImageCanvas imageCanvas;
	private BufferedImage image;
	
	private boolean done;
	private String len_hdr;
	private double maxFPS;
	private int timeToWait;
	
	/**
	 * Create a ImageDownload that will connect to host on port and will set
	 * the maximum frames per second to fps with the image canvis as the 
	 * primary display   
	 * 
	 * @param conn a senscor connection to the sensor's server
	 * @param fps the maximum rate of image capture
	 * @param imagecanvas the canvis to display the image on
	 */
	public VisionConnection(String host, int port, double fps, ImageCanvas imagecanvas)
	{
		len_hdr = "content-length: ";
		imageCanvas = imagecanvas;
		setMaxFPS(fps);
		setHost(host);
		setPort(port);
	}

	/**
	 * Set the maximum capture rate
	 * @param d
	 */
	public void setMaxFPS(double d)
	{
		// prevent a fps rate that is less than 0
		if(d > 0.0D)
		{
			maxFPS = d;
			timeToWait = (int)(1000D / d);
		}
	}

	/**
	 * Get the maximum capture rate
	 * 
	 * @return
	 */
	public double getMaxFPS()
	{
		return maxFPS;
	}
	
	/**
	 * Returns the last captured image
	 * @return
	 */
	public BufferedImage getImage() {
		return image;
	}

	/**
	 * Disconnects from the camera server
	 */
	public void disconnect()
	{
		done = true;
		super.disconnect();
	}

	/**
	 * Returns the status of the connection to the camera server
	 * 
	 * @return true if the connection was successful, false otherwise
	 */
	public boolean isConnected()
	{
		return !done;
	}

	/**
	 * Starts capturing images from the camera server at the set fps rate
	 */
	public void run()
	{
		done = false;
		
		try { 
			connect();
		} catch (Exception e) {
			imageCanvas.couldNotConnect();
			imageCanvas.setConnected(1);
			return;
		}
		
		imageCanvas.setConnected(0);
		try
		{
			PrintWriter printwriter = new PrintWriter(getConnection().getOutputStream());
			java.io.InputStream inputstream = getConnection().getInputStream();
			BufferedInputStream bufferedinputstream = new BufferedInputStream(inputstream);
			byte abyte0[] = new byte[1024];
			while(!done) 
			{
				long l1 = System.currentTimeMillis();
				printwriter.println("GET / WCS/0.50");
				printwriter.flush();
				for(int i = 0; (abyte0[i] = (byte)bufferedinputstream.read()) != -1; i++)
					if(i > 0 && abyte0[i] == 10 && abyte0[i - 1] == 10)
						break;

				String s = (new String(abyte0)).toLowerCase();
				int j = s.indexOf(len_hdr) + len_hdr.length();
				int k = s.indexOf("\n", j);
				s = s.substring(j, k);
				int i1 = Integer.parseInt(s);
				if(i1 < 0)
					break;
				byte abyte1[] = new byte[i1];
				int j1 = 0;
				int k1 = 0;
				while((j1 = bufferedinputstream.read(abyte1, k1, i1 - k1)) != -1) 
				{
					k1 += j1;
					if(k1 == i1)
						break;
					wait(10);
				}
				if(k1 != i1)
				{
					System.out.println("Did not read everything!");
					break;
				}
				
				image = javax.imageio.ImageIO.read(new ByteArrayInputStream(abyte1));
				
				
				if(imageCanvas != null) {
					imageCanvas.setImage(image);
					
					
				}
				
				long l3 = System.currentTimeMillis();
				if((long)timeToWait - (l3 - l1) > 0L) {
					wait((int)((long)timeToWait - (l3 - l1)));
				}				
			}
		}
		catch(IOException ioexception)
		{
			ioexception.printStackTrace();
			imageCanvas.setConnected(1);
		}
		finally
		{
			disconnect();
			imageCanvas.setConnected(2);
		}
	}

	/**
	 * @deprecated
	 * @param abyte0
	 * @return
	 */
	@SuppressWarnings("unused")
	private int FourBytesToInt(byte abyte0[])
	{
		int i = 0;
		int j = 0;
		i += unsignedByteToInt(abyte0[j++]);
		i += unsignedByteToInt(abyte0[j++]) << 8;
		i += unsignedByteToInt(abyte0[j++]) << 16;
		i += unsignedByteToInt(abyte0[j++]) << 24;
		return i;
	}

	private static int unsignedByteToInt(byte byte0)
	{
		return byte0 & 0xff;
	}

	private void wait(int i)
	{
		try
		{
			Thread.sleep(i);
			return;
		}
		catch(InterruptedException _ex)
		{
			return;
		}
	}
}
