package edu.wpi.ari.glados.vision.vidar;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.Raster;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.imageio.ImageIO;
/**The Detector function implements matrix convolution operations needed for
 * the implementation of the ARI simple vision system. 
 * @author  Zack A. Kleinfeld
 * @Note The convolutions can be modularly replaced with any function with the same calling sequence.
 * The functions have no side effects.
 */
public class Detector {
	public Point [] filter(BufferedImage lhs, BufferedImage rhs){
		
		return new Point [1];
	}
	
	/**Loads an image from a testing file
	 * @param f file to read
	 * @return a BufferedImage from the file.
	 */
	public BufferedImage loadImage(File f){
		BufferedImage i = null;
		try {
			i = ImageIO.read(f);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return i;
	}
	
	/**A matrix operation meant to eliminate pixels that arn't on an edge.
	 * @param src Source Image
	 * @param dest Destination Image
	 */
	public void edgeDetect(BufferedImage src, BufferedImage dest){
		float[] elements = { 0.0f, -1.0f, 0.0f, 
                -1.0f, 4.f, -1.0f, 
                0.0f, -1.0f, 0.0f}; 
		Kernel kernel = new Kernel(3, 3, elements); 
		ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null); 
		cop.filter(src, dest); 
	}
	/**A matrix operation meant to eliminate pixels that arn't in a line.
	 * @param src Source Image
	 * @param dest Destination Image
	 */
	public void lineDetect(BufferedImage src, BufferedImage dest){
		float[] elements = { -2.0f, 1.0f, -2.0f, 
                -2.0f, 1.0f, -2.0f, 
                -2.0f, 1.0f, -2.0f}; 
		Kernel kernel = new Kernel(3, 3, elements); 
		ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null); 
		cop.filter(src, dest); 
	}
	/**A matrix operation meant to eliminate pixels that don't have neighbors.
	 * @param src Source Image
	 * @param dest Destination Image
	 */
	public void highDetect(BufferedImage src, BufferedImage dest){
		float[] elements = { -1.0f, -1.0f, -1.0f, 
                -1.0f, 1.0f, -1.0f, 
                -1.0f, -1.0f, -1.0f}; 
		Kernel kernel = new Kernel(3, 3, elements); 
		ConvolveOp cop = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null); 
		cop.filter(src, dest); 
	}
	
	
	/**Feature Detection function to count, enumerate, and return the remaining pixels in an image,
	 * which should be the features.
	 * @param src Image which has been processed to eliminate most pixels.
	 * @param sample The color channel to detect on.
	 * @param threshhold The brightness of the channel needed to qualify as a feature.
	 * @return A Vector of Points representing features.
	 */
	public Vector<Point> featureDetect(BufferedImage src,
			int sample, int threshhold){
		Vector<Point> results = new Vector<Point>();
		Raster grid = src.getData();
		for (int x=1; x<grid.getWidth()-1; x++) { //Avoid Borders
			for(int y=1; y<grid.getHeight()-1; y++){ //Avoid Borders
				if (grid.getSample(x, y, sample) >= threshhold){
					results.add(new Point(x, y));
				}
			}
		}
		return results;
		
	}
	
}
