
package edu.wpi.ari.glados.ins;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import javax.swing.Timer;


public class INSServer extends Thread implements INSController, INSDataObserver{
	public static final int PORT = 4445;

	private SpotListener listener = null;

	private ServerSocket server;
	private Socket client;

	private DataOutputStream oos;
	private GINSServer gui;
	private AccelerometerListener acl;
	
	private boolean run;
	
    public static void main(String args[]) {
        new INSServer();
    }
	
	public INSServer() {

		
		gui = new GINSServer();
		gui.setINSControler(this);
		
		run = true;
		
		try {
			server = new ServerSocket(PORT);
			System.out.println("Server Started on port: " + PORT);
		} catch (IOException e) {
			System.out.println("Server could not start " + PORT);
		}
		
		listener = new SpotListener(this);
		listener.start();
		
		this.start();
	}
	
	public void run() {
		gui.setConnected(true);
		
		while(run) {
			try {
				System.out.println("Waiting for connections.");
				client = server.accept();
				oos = new DataOutputStream(client.getOutputStream());
	
				System.out.println("Accepted a connection from: " + client.getInetAddress());
				
			} catch (IOException e) {
				System.err.println("Accept failed: " + PORT);
			}
		}
	}
	
	public void setConnected(boolean c) {
		gui.setConnected(c);
	}

	public void ping() {
		listener.doPing();
	}

	public void quit() {
		listener.doQuit();
		System.exit(0);
	}

	public void update(String address, long time, double x, double y, double z, int yaw) {

		if(client!= null && client.isConnected()) {
			try {
				oos.writeLong(time);
				oos.writeDouble(x);
				oos.writeDouble(y);
				oos.writeDouble(z);
				oos.writeInt(yaw);
				oos.flush();
			} catch (IOException e) {
				System.err.println("Error sending data packet to Glados");
			}
		}
		
	}

	public void reconnect() {
		listener.reconnect();
	}

	public void startSpot() {
		System.out.println("Starting Spot");
		listener.doCalibrate();
		listener.doSendData(true);
	}
}
