package edu.wpi.rbreznak.jtank.controller;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;

import wiiremotej.ButtonMap;
import wiiremotej.IRLight;
import wiiremotej.WiiRemote;
import wiiremotej.WiiRemoteExtension;
import wiiremotej.WiiRemoteJ;
import wiiremotej.event.WRAccelerationEvent;
import wiiremotej.event.WRButtonEvent;
import wiiremotej.event.WRExtensionEvent;
import wiiremotej.event.WRIREvent;
import wiiremotej.event.WRNunchukExtensionEvent;
import wiiremotej.event.WRStatusEvent;
import wiiremotej.event.WiiRemoteAdapter;
import wiiremotej.event.WiiRemoteDiscoveredEvent;
import wiiremotej.event.WiiRemoteDiscoveryListener;
import wiiremotej.event.WiiRemoteListener;

/**
 * Implements WiiRemoteListener and acts as a general test class. Note that you
 * can ignore the main method pretty much, as it mostly has to do with the
 * graphs and GUIs. At the very end though, there's an example of how to connect
 * to a remote and how to prebuffer audio files.
 * 
 * @author Michael Diamond
 * @version 1/05/07
 */

public class WiiControler extends WiiRemoteAdapter  implements WiiRemoteDiscoveryListener, WiiRemoteListener{
	private static boolean status = false;

	private WiiRemote remote;
	
	private boolean paused = false;
	
	private Display disp = new Display();
	
	public static void main(String [] args) {
		WiiControler wc = new WiiControler();
		wc.connect();
	}
	
	public WiiControler() {
		JFrame frame = new JFrame("WiiMote Demo");
		JPanel panel = new JPanel(new BorderLayout());
		
		JButton conn = new JButton("Reconnect");
		conn.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				connect();
			}
			
		});
		
		panel.add(disp, BorderLayout.CENTER);
		panel.add(conn, BorderLayout.NORTH);
		
		frame.setContentPane(panel);
		frame.setVisible(true);
		frame.setSize(new Dimension(600, 600));
	}
	
	public void connect() {

		WiiRemoteJ.setConsoleLoggingAll();

		try {

			// Find and connect to a Wii Remote
			WiiRemote remote = WiiRemoteJ.findRemote();
			remote.addWiiRemoteListener(this);
			remote.setAccelerometerEnabled(true);
			remote.setSpeakerEnabled(true);
			remote.setIRSensorEnabled(true, WRIREvent.BASIC);
			remote.setLEDIlluminated(0, true);

			remote.getButtonMaps().add(
					new ButtonMap(WRButtonEvent.HOME, ButtonMap.NUNCHUK,
							WRNunchukExtensionEvent.C,
							new int[] { java.awt.event.KeyEvent.VK_CONTROL },
							java.awt.event.InputEvent.BUTTON1_MASK, 0, -1));

		} catch (Exception e) {
			e.printStackTrace();
		}
		
		System.out.println("WiiMote Conntected.");
	}

	public void disconnected() {
		System.out.println("Remote disconnected.");
	}

	public void statusReported(WRStatusEvent evt) {
		System.out.println("Batt: " + (double) evt.getBatteryLevel() / 2);
		System.out.println("Cont: " + evt.isContinuousEnabled());
		System.out.println("Rem:  " + remote.isContinuousEnabled());
	}

	public void IRInputReceived(WRIREvent evt) {
		for (IRLight light : evt.getIRLights()) {
			if (light != null) {
				//System.out.println("X: " + light.getX());
				//System.out.println("Y: " + light.getY());
			}
		}

	}

	public void accelerationInputReceived(WRAccelerationEvent evt) {

		double x = evt.getXAcceleration();
		double y = evt.getYAcceleration();
		//int z = (int) (evt.getZAcceleration() / 5 * 300);

		if(status) { 
			disp.setX(x);
			disp.setY(y);
			disp.repaint();
		} else {
			disp.setX(0);
			disp.setY(0);
			disp.repaint();
		}
	}

	public void extensionInputReceived(WRExtensionEvent evt) {

	}

	public void extensionConnected(WiiRemoteExtension extension) {
		try {
			remote.setExtensionEnabled(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void extensionPartiallyInserted() {

	}

	public void extensionUnknown() {
		
	}

	public void extensionDisconnected(WiiRemoteExtension extension) {
		
	}

	public void wiiRemoteDiscovered(WiiRemoteDiscoveredEvent evt) {
		evt.getWiiRemote().addWiiRemoteListener(this);
	}

	public void findFinished(int numberFound) {
		System.out.println("Found " + numberFound + " remotes!");
	}
	
	public void buttonInputReceived(WRButtonEvent evt) {
		int x = 0;
		int y = 0;
		
		try {
			if(evt.isPressed(WRButtonEvent.UP)) {
				y = 20;
			} else if(evt.isPressed(WRButtonEvent.DOWN)) {
				y = -20;
			} else if(evt.isPressed(WRButtonEvent.LEFT)) {
				x = 20;
			} else if(evt.isPressed(WRButtonEvent.RIGHT)) { 
				x = -20;
			} else if(evt.isPressed(WRButtonEvent.B)) {
				status = true;
			} else if(evt.wasReleased(WRButtonEvent.B)){
				status = false;
			}
			
			System.out.println("Foo: " + x + "\t" + y);
			
			final Timer t = new Timer(200, new ActionListener() {

				public void actionPerformed(ActionEvent a) {
					disp.setX(0);
					disp.setY(0);
					disp.repaint();
					((Timer) a.getSource()).stop();
					paused = false;
				} 
				
			});
				
			if(!paused) {
				disp.setX(x);
				disp.setY(y);
				disp.repaint();
				t.start();
			} 
			
			paused = true;

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}